"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
const interfaces_1 = require("./interfaces");
const Boom = require("boom");
function AppDecorator(config) {
    return (target, propertyKey, descriptor) => {
        Reflect.defineMetadata('hapiour:config', config, target);
    };
}
exports.AppDecorator = AppDecorator;
function InjectDecorator(Modules) {
    return (target, propertyKey, descriptor) => {
        Reflect.defineMetadata('hapiour:modules', Modules, target);
    };
}
exports.InjectDecorator = InjectDecorator;
function PluginsDecorator(Plugins) {
    return (target, propertyKey, descriptor) => {
        Reflect.defineMetadata('hapiour:plugins', Plugins, target);
    };
}
exports.PluginsDecorator = PluginsDecorator;
function PluginDecorator(attributes) {
    return (target, propertyKey, descriptor) => {
        Reflect.defineMetadata('hapiour:attributes', attributes, target);
    };
}
exports.PluginDecorator = PluginDecorator;
function PluginConfiguratorDecorator(Plugin) {
    return (target, propertyKey, descriptor) => {
        Reflect.defineMetadata('hapiour:register', Plugin, target);
    };
}
exports.PluginConfiguratorDecorator = PluginConfiguratorDecorator;
function ModuleDecorator(config) {
    return (target, propertyKey, descriptor) => {
        Reflect.defineMetadata('hapiour:config', config, target);
    };
}
exports.ModuleDecorator = ModuleDecorator;
function RouteDecorator(route) {
    return (target, propertyKey, descriptor) => {
        let routes = [];
        if (Reflect.hasMetadata('hapiour:routes', target.constructor)) {
            routes = Reflect.getMetadata('hapiour:routes', target.constructor);
        }
        route.handler = descriptor.value;
        routes.push(route);
        Reflect.defineMetadata('hapiour:routes', routes, target.constructor);
    };
}
exports.RouteDecorator = RouteDecorator;
function getArgs(func) {
    let args = func.toString().match(/\(([^)]*)\)/)[1];
    return args.split(',').map(function (arg) {
        return arg.replace(/\/\*.*\*\//, '').trim();
    }).filter(function (arg) {
        return arg;
    });
}
function MappingsDecorator(target, propertyKey, index, type, parse_it) {
    let args = getArgs(target[propertyKey]);
    let mapping_types = [];
    if (Reflect.hasMetadata('hapiour:mappings', target[propertyKey])) {
        mapping_types = Reflect.getOwnMetadata('hapiour:mappings', target[propertyKey]) || [];
    }
    mapping_types.push({
        param_name: args[index],
        type: type,
        parse_it: parse_it
    });
    Reflect.defineMetadata('hapiour:mappings', mapping_types, target[propertyKey]);
}
exports.MappingsDecorator = MappingsDecorator;
function ParamNumberDecorator(target, propertyKey, index) {
    MappingsDecorator(target, propertyKey, index, interfaces_1.MType.param, (v) => {
        if (!v) {
            return undefined;
        }
        let num = parseInt(v);
        if (isNaN(num)) {
            throw Boom.badData(`${v} is not a Number`, v);
        }
        return num;
    });
}
exports.ParamNumberDecorator = ParamNumberDecorator;
function RequestAttributeDecorator(target, propertyKey, index) {
    MappingsDecorator(target, propertyKey, index, interfaces_1.MType.req_attr);
}
exports.RequestAttributeDecorator = RequestAttributeDecorator;
function ParamDecorator(target, propertyKey, index) {
    MappingsDecorator(target, propertyKey, index, interfaces_1.MType.param);
}
exports.ParamDecorator = ParamDecorator;
function RequireRequestAttributeDecorator(target, propertyKey, index) {
    MappingsDecorator(target, propertyKey, index, interfaces_1.MType.require_req_attr);
}
exports.RequireRequestAttributeDecorator = RequireRequestAttributeDecorator;
function RequestDecorator(target, propertyKey, index) {
    MappingsDecorator(target, propertyKey, index, interfaces_1.MType.request);
}
exports.RequestDecorator = RequestDecorator;
function ReplyAttributeDecorator(target, propertyKey, index) {
    MappingsDecorator(target, propertyKey, index, interfaces_1.MType.reply);
}
exports.ReplyAttributeDecorator = ReplyAttributeDecorator;
//# sourceMappingURL=decorators.js.map